/*
 * Copyright (c) 2014 Patrick P. Frey
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#ifndef _STRUS_RPC_CONST_CONSTRUCTOR_HPP_INCLUDED
#define _STRUS_RPC_CONST_CONSTRUCTOR_HPP_INCLUDED
#include <cstdlib>
#include <cstring>
#include <vector>
#include <stdexcept>

namespace strus {

class ConstConstructor
{
public:
	ConstConstructor(){}
	~ConstConstructor()
	{
		std::vector<Value>::iterator ai = m_longliving_ar.begin(), ae = m_longliving_ar.end();
		for (; ai != ae; ++ai)
		{
			std::free( ai->value);
		}
	}

	void reset()
	{
		std::vector<Value>::iterator ai = m_ar.begin(), ae = m_ar.end();
		for (; ai != ae; ++ai)
		{
			std::free( ai->value);
		}
		m_ar.clear();
	}

	const char* getCharp( const char* ptr)
	{
		return (const char*)get( ptr, std::strlen(ptr)+1);
	}
	const char** getCharpp( const char** ptr)
	{
		char const** pp = ptr;
		while (*pp)++pp;
		return (const char**)get( ptr, ((pp-ptr)+1)*sizeof(char*));
	}

	const void* get( const void* ptr, std::size_t size)
	{
		m_ar.push_back( Value::copy( ptr, size));
		return m_ar.back().value;
	}
	const void* getLongLiving( const void* ptr, std::size_t size)
	{
		m_longliving_ar.push_back( Value::copy( ptr, size));
		return m_longliving_ar.back().value;
	}

private:
	struct Value
	{
		void* value;
		std::size_t size;

		~Value(){}

		Value()
			:value(0),size(0){}
		Value( const Value& o)
			:value(o.value),size(o.size){}
		Value( void* value_, std::size_t size_)
			:value(value_),size(size_)
		{}

		static Value copy( const void* ptr_, std::size_t size_)
		{
			void* ptr = std::malloc(size_);
			if (!ptr) throw std::bad_alloc();
			std::memcpy( ptr, ptr_, size_);
			return Value( ptr, size_);
		}
	};

	std::vector<Value> m_ar;
	std::vector<Value> m_longliving_ar;
};

}//namespace
#endif
